/**
 * 
 */
package de.bablokb.epm.views;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.ISharedImages;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ViewPart;

import de.bablokb.epm.utils.DirListProvider;

/**
 * @author bablokb
 * 
 */
public class DirListView extends ViewPart {
	private String iDirname = null;
private IMemento iMemento = null;
	private ListViewer iViewer;

	private IAction iOpenAction;

	private IAction iClearAction;

	public final static String VIEW_ID = "ListView";

	/**
	 * 
	 */
	public DirListView() {
		super();
	}

	
	
	public void init(IViewSite site, IMemento memento) throws PartInitException {
		iMemento = memento;
		super.init(site, memento);
	}

	public void init(IViewSite site) throws PartInitException {
		// TODO Auto-generated method stub
		super.init(site);
	}



	public void createPartControl(Composite parent) {
		iViewer = new ListViewer(parent);
		iViewer.setContentProvider(new DirListProvider());
		iViewer.setLabelProvider(new LabelProvider());
		restoreState();
		iViewer.setInput(iDirname);
		getSite().setSelectionProvider(iViewer);
		makeActions();
		hookContextMenu();
		contributeToActionBars();
	}

	private void restoreState() {
		if (iMemento == null) {
			if (iDirname == null) {
				iDirname = System.getProperty("user.home");
			}
			return;
		}
		IMemento dirname = iMemento.getChild("directory");
		if (dirname != null) {
			iDirname = dirname.getID();
		}
	}

	private void hookContextMenu() {
		MenuManager menuMgr = new MenuManager("#PopupMenu");
		menuMgr.setRemoveAllWhenShown(true);
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager manager) {
				DirListView.this.fillContextMenu(manager);
			}
		});
		Menu menu = menuMgr.createContextMenu(iViewer.getControl());
		iViewer.getControl().setMenu(menu);
		getSite().registerContextMenu(menuMgr, iViewer);
	}

	private void contributeToActionBars() {
		IActionBars bars = getViewSite().getActionBars();
		fillLocalPullDown(bars.getMenuManager());
		fillLocalToolBar(bars.getToolBarManager());
	}

	private void fillLocalPullDown(IMenuManager manager) {
		manager.add(iOpenAction);
		manager.add(iClearAction);
	}

	private void fillContextMenu(IMenuManager manager) {
		manager.add(iOpenAction);
		manager.add(iClearAction);
		// Other plug-ins can contribute there actions here
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillLocalToolBar(IToolBarManager manager) {
		manager.add(iOpenAction);
		manager.add(iClearAction);
	}

	private void makeActions() {
		iOpenAction = new Action() {
			public void run() {
				DirectoryDialog dirChooser = new DirectoryDialog(getSite()
						.getShell(), SWT.OPEN);
				dirChooser.setText("Open image directory");
				dirChooser.setFilterPath(iDirname);
				iDirname = dirChooser.open();
				if (iDirname != null) {
					iViewer.setInput(iDirname);
				}
			}
		};
		iOpenAction.setText("Select directory");
		iOpenAction.setToolTipText("search files in this directory");
		iOpenAction.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_OBJ_FOLDER));

		iClearAction = new Action() {
			public void run() {
				iViewer.getList().deselectAll();
				iViewer.refresh();
			}
		};
		iClearAction.setText("Clear selection");
		iClearAction.setToolTipText("deselect all");
		iClearAction.setImageDescriptor(PlatformUI.getWorkbench()
				.getSharedImages().getImageDescriptor(
						ISharedImages.IMG_TOOL_UNDO));
	}

	public void setFocus() {
		iViewer.getControl().setFocus();
	}

	public void saveState(IMemento memento) {
		memento.createChild("directory",iDirname);
		super.saveState(memento);
	}	
}